<?php

namespace Drupal\cmlstarter_demo\Utility;

use Drupal\migrate\Plugin\migrate\source\SourcePluginBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Source for Plugins.
 */
class MigrationsSourceBase extends SourcePluginBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);
    // SourcePlugin Settings.
    $this->trackChanges = TRUE;

    $uipage = $this->uiPage();
    $config = self::accessProtected($migration, 'pluginDefinition');
    $this->config = $config['process'];
    // Debug switcher.
    $this->debug = FALSE;
    $this->rows = [];
    if ($this->uipage && $this->debug) {
      if (\Drupal::moduleHandler()->moduleExists('devel')) {
        dsm("{$plugin_id}: ProcessMapping & Rows");
        // dsm($this->config);
        // dsm($rows);
      }
    }
  }

  /**
   * Get Rows.
   */
  public function getRows() {
    $rows = [];
    $this->rows = $rows;
    return $rows;
  }

  /**
   * UiPage.
   */
  public function uiPage() {
    $uipage = FALSE;
    $statuspage = FALSE;
    if (\Drupal::routeMatch()->getRouteName() == "entity.migration.list") {
      $uipage = TRUE;
    }
    if (\Drupal::routeMatch()->getRouteName() == "cmlmigrations.status") {
      $statuspage = TRUE;
    }
    $this->uipage = $uipage;
    $this->statuspage = $statuspage;
    return $uipage;
  }

  /**
   * Access Protected Obj Property.
   */
  public static function accessProtected($obj, $prop) {
    $reflection = new \ReflectionClass($obj);
    $property = $reflection->getProperty($prop);
    $property->setAccessible(TRUE);
    return $property->getValue($obj);
  }

  /**
   * {@inheritdoc}
   */
  public function initializeIterator() {
    $rows = $this->getRows();
    return new \ArrayIterator($rows);
  }

  /**
   * Allows class to decide how it will react when it is treated like a string.
   */
  public function __toString() {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function getIDs() {
    return [
      'id' => [
        'type' => 'string',
        'alias' => 'id',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $fields = [
      'id' => $this->t('ID'),
    ];
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function count($refresh = FALSE) {
    $source = $this->getContent(TRUE);
    return count($source);
  }

  /**
   * Exec.
   */
  public function getContent($dir, $count = FALSE) {
    $module_handler = \Drupal::service('module_handler');
    $path = DRUPAL_ROOT . "/" . $module_handler->getModule('cmlstarter_demo')->getPath();

    $source = [];
    // Scandir.
    $files = scandir("$path/content/{$dir}");
    if (isset($files[2])) {
      foreach ($files as $f) {
        if (substr($f, -4) == '.yml') {
          if (!$count) {
            $source[] = Yaml::parse(file_get_contents("$path/content/{$dir}/$f"));
          }
          else {
            $source[] = $f;
          }
        }
      }
    }
    return $source;
  }

  /**
   * Ensures the existence of a file.
   */
  public function ensureFiles($filenames, $dir) {
    $file_storage = \Drupal::entityManager()->getStorage('file');
    $raw = "https://raw.githubusercontent.com/synapse-studio/helper/master";
    $tmp = "/tmp";
    $result = [];
    if (is_string($filenames)) {
      $filenames = [$filenames];
    }

    if (is_array($filenames) && !empty($filenames)) {
      $fs = \Drupal::service('file_system');
      $directory = "public://cmlstarter-demo/{$dir}/";
      if (!file_exists($fs->realpath($directory))) {
        \Drupal::service('file_system')->mkdir($directory, NULL, TRUE);
      }
      foreach ($filenames as $fname) {
        $filename = "{$dir}-{$fname}";
        $files = $file_storage->loadByProperties(['filename' => $filename]);
        $file = reset($files);
        if (!$this->uiPage() && !$file) {
          $f = file_get_contents("{$raw}/content/{$dir}/{$fname}");
          $destination = "{$directory}{$filename}";
          $uri = file_unmanaged_save_data($f, $destination, FILE_EXISTS_REPLACE);
          $file = $file_storage->create([
            'filename' => $destination,
            'uri' => $uri,
            'status' => 1,
          ]);
          $file->save();
        }
        if (is_object($file)) {
          $id = $file->id();
          $result[$id] = ['target_id' => $id];
        }
      }
    }
    return $result;
  }

  /**
   * Ensures the existence of a term id.
   */
  public function ensureTerm($uuids) {
    $result = [];
    if (is_string($uuids)) {
      $uuids = [$uuids];
    }
    if (is_array($uuids)) {
      foreach ($uuids as $uuid) {
        $term = \Drupal::service('entity.repository')->loadEntityByUuid('taxonomy_term', $uuid);
        if (is_object($term)) {
          $id = $term->id();
          $result[$id] = ['target_id' => $id];
        }
      }
    }
    return $result;
  }

}
